/*
 * inetd/data.c --
 *
 *      This module reads service definitions from a configuration
 *      file and adds service records to the global list of configured
 *      services.
 */

#define _POSIX_C_SOURCE 201112L
#define _DEFAULT_SOURCE
#define _DARWIN_C_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <assert.h>

#include "inetd.h"

inetd_service_t *inetd_services = NULL;
inetd_client_t *inetd_clients = NULL;

/*
 * A wrapper around malloc() to handle memory allocation failures.  We
 * also initialize the allocated memory to zeros.
 */

static void*
xmalloc(size_t size)
{
    void *p = malloc(size);
    if (!p) {
	syslog(LOG_ERR, "malloc() failed to allocate memory - exiting");
	exit(EXIT_FAILURE);
    }
    memset(p, 0, size);
    return p;
}

/*
 * A wrapper around strdup() to handle memory allocation failures.
 */

static char*
xstrdup(const char *s)
{
    char *p = strdup(s);
    if (!p) {
	syslog(LOG_ERR, "strdup() failed to allocate memory - exiting");
	exit(EXIT_FAILURE);
    }
    return p;
}

inetd_service_t*
inetd_new_service()
{
    inetd_service_t *sp;

    sp = (inetd_service_t *) xmalloc(sizeof(inetd_service_t));
    sp->next = inetd_services;
    inetd_services = sp;
    return sp;
}

void
inetd_del_service(inetd_service_t *sp)
{
    inetd_service_t **pp, *d;
    int i;
    
    assert(sp);
	
    for (pp = &inetd_services; *pp; pp = &((*pp)->next)) {
	if ((*pp) == sp) break;
    }

    if (pp) {
	d = *pp;
	(*pp) = (*pp)->next;
	if (d->port) free(d->port);
	if (d->name) free(d->name);
	for (i=0; i < INETD_MAX_PARAM; i++) {
	    if (d->params[i]) free(d->params[i]);
	}
	free(d);
    }
}

inetd_client_t*
inetd_new_client(inetd_service_t *sp)
{
    inetd_client_t *cp;

    assert(sp);

    cp = (inetd_client_t *) xmalloc(sizeof(inetd_client_t));
    cp->service = sp;

    cp->next = inetd_clients;
    inetd_clients = cp;
    return cp;
}

void
inetd_del_client(inetd_client_t *cp)
{
    inetd_client_t **pp, *d;
    
    assert(cp);
    
    for (pp = &inetd_clients; *pp; pp = &((*pp)->next)) {
	if ((*pp) == cp) break;
    }

    if (pp) {
	d = *pp;
	(*pp) = (*pp)->next;
	free(d);
    }
}

/*
 * Read configuration from a file and return the list of configuration
 * lines read. The list may be in reverse order (since we are lazy).
 */

void
inetd_read_config(const char* filename)
{
    char line[1024], *token;
    inetd_service_t *sp = NULL;
    FILE *file;
    int i;
	
    file = fopen(filename, "r");
    if (file == NULL) {
	syslog(LOG_ERR, "cannot open configuration file '%s': %m", filename);
	exit(1);
    }

    while (fgets(line, sizeof(line), file)) {
	sp = inetd_new_service();
	token = strtok(line," \n\t");
	sp->port = xstrdup(token);
	token = strtok(NULL, " \n\t");
	sp->name = xstrdup(token);
	i = 0;
	while ((token = strtok(NULL, " \n\t")) && i < INETD_MAX_PARAM-1) {
	    sp->params[i++] = xstrdup(token);
	}
	sp->params[i] = NULL;	/* params have to be NULL terminated */
    }
    
    (void) fclose(file);
}

